@@ -12,19 +12,20 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from ctypes import c_bool
 import multiprocessing
 import os
 from six.moves import http_client as httplib
 import socket
 import sys
+import threading
+import time
 
 import cotyledon
 import flask
 from pyroute2.ipdb import transactional
-import retrying
 
 import os_vif
-from os_vif import objects as obj_vif
 from os_vif.objects import base
 from oslo_concurrency import lockutils
 from oslo_config import cfg
@@ -32,113 +33,28 @@ from oslo_log import log as logging
 from oslo_serialization import jsonutils
 
 from kuryr_kubernetes import clients
-from kuryr_kubernetes.cni import api
-from kuryr_kubernetes.cni.binding import base as b_base
 from kuryr_kubernetes.cni import handlers as h_cni
-from kuryr_kubernetes.cni import utils
+from kuryr_kubernetes.cni import health
+from kuryr_kubernetes.cni.plugins import k8s_cni_registry
+from kuryr_kubernetes.cni import utils as cni_utils
 from kuryr_kubernetes import config
 from kuryr_kubernetes import constants as k_const
 from kuryr_kubernetes import exceptions
 from kuryr_kubernetes import objects
+from kuryr_kubernetes import utils
 from kuryr_kubernetes import watcher as k_watcher
 
 LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
-RETRY_DELAY = 1000  # 1 second in milliseconds
-
-# TODO(dulek): Another corner case is (and was) when pod is deleted before it's
-#              annotated by controller or even noticed by any watcher. Kubelet
-#              will try to delete such vif, but we will have no data about it.
-#              This is currently worked around by returning succesfully in case
-#              of timing out in delete. To solve this properly we need to watch
-#              for pod deletes as well.
-
-
-class K8sCNIRegistryPlugin(api.CNIPlugin):
-    def __init__(self, registry):
-        self.registry = registry
-
-    def _get_name(self, pod):
-        return pod['metadata']['name']
-
-    def add(self, params):
-        vif = self._do_work(params, b_base.connect)
-
-        pod_name = params.args.K8S_POD_NAME
-        # NOTE(dulek): Saving containerid to be able to distinguish old DEL
-        #              requests that we should ignore. We need a lock to
-        #              prevent race conditions and replace whole object in the
-        #              dict for multiprocessing.Manager to notice that.
-        with lockutils.lock(pod_name, external=True):
-            d = self.registry[pod_name]
-            d['containerid'] = params.CNI_CONTAINERID
-            self.registry[pod_name] = d
-            LOG.debug('Saved containerid = %s for pod %s',
-                      params.CNI_CONTAINERID, pod_name)
-
-        # Wait for VIF to become active.
-        timeout = CONF.cni_daemon.vif_annotation_timeout
-
-        # Wait for timeout sec, 1 sec between tries, retry when vif not active.
-        @retrying.retry(stop_max_delay=timeout * 1000, wait_fixed=RETRY_DELAY,
-                        retry_on_result=lambda x: not x.active)
-        def wait_for_active(pod_name):
-            return base.VersionedObject.obj_from_primitive(
-                self.registry[pod_name]['vif'])
-
-        vif = wait_for_active(pod_name)
-        if not vif.active:
-            raise exceptions.ResourceNotReady(pod_name)
-
-        return vif
-
-    def delete(self, params):
-        pod_name = params.args.K8S_POD_NAME
-        try:
-            reg_ci = self.registry[pod_name]['containerid']
-            LOG.debug('Read containerid = %s for pod %s', reg_ci, pod_name)
-            if reg_ci and reg_ci != params.CNI_CONTAINERID:
-                # NOTE(dulek): This is a DEL request for some older (probably
-                #              failed) ADD call. We should ignore it or we'll
-                #              unplug a running pod.
-                LOG.warning('Received DEL request for unknown ADD call. '
-                            'Ignoring.')
-                return
-        except KeyError:
-            pass
-        self._do_work(params, b_base.disconnect)
-
-    def _do_work(self, params, fn):
-        pod_name = params.args.K8S_POD_NAME
-
-        timeout = CONF.cni_daemon.vif_annotation_timeout
-
-        # In case of KeyError retry for `timeout` s, wait 1 s between tries.
-        @retrying.retry(stop_max_delay=timeout * 1000, wait_fixed=RETRY_DELAY,
-                        retry_on_exception=lambda e: isinstance(e, KeyError))
-        def find():
-            return self.registry[pod_name]
-
-        try:
-            d = find()
-            pod = d['pod']
-            vif = base.VersionedObject.obj_from_primitive(d['vif'])
-        except KeyError:
-            raise exceptions.ResourceNotReady(pod_name)
-
-        fn(vif, self._get_inst(pod), params.CNI_IFNAME, params.CNI_NETNS)
-        return vif
-
-    def _get_inst(self, pod):
-        return obj_vif.instance_info.InstanceInfo(
-            uuid=pod['metadata']['uid'], name=pod['metadata']['name'])
+HEALTH_CHECKER_DELAY = 5
 
 
 class DaemonServer(object):
-    def __init__(self, plugin):
+    def __init__(self, plugin, healthy):
         self.ctx = None
         self.plugin = plugin
-
+        self.healthy = healthy
+        self.failure_count = multiprocessing.Value('i', 0)
         self.application = flask.Flask('kuryr-daemon')
         self.application.add_url_rule(
             '/addNetwork', methods=['POST'], view_func=self.add)
@@ -148,7 +64,7 @@ class DaemonServer(object):
                         'Connection': 'close'}
 
     def _prepare_request(self):
-        params = utils.CNIParameters(flask.request.get_json())
+        params = cni_utils.CNIParameters(flask.request.get_json())
         LOG.debug('Received %s request. CNI Params: %s',
                   params.CNI_COMMAND, params)
         return params
@@ -157,6 +73,7 @@ class DaemonServer(object):
         try:
             params = self._prepare_request()
         except Exception:
+            self._check_failure()
             LOG.exception('Exception when reading CNI params.')
             return '', httplib.BAD_REQUEST, self.headers
 
@@ -164,10 +81,12 @@ class DaemonServer(object):
             vif = self.plugin.add(params)
             data = jsonutils.dumps(vif.obj_to_primitive())
         except exceptions.ResourceNotReady as e:
+            self._check_failure()
             LOG.error("Timed out waiting for requested pod to appear in "
                       "registry: %s.", e)
             return '', httplib.GATEWAY_TIMEOUT, self.headers
         except Exception:
+            self._check_failure()
             LOG.exception('Error when processing addNetwork request. CNI '
                           'Params: %s', params)
             return '', httplib.INTERNAL_SERVER_ERROR, self.headers
@@ -209,22 +128,33 @@ class DaemonServer(object):
             raise
 
         try:
-            self.application.run(address, port,
+            self.application.run(address, port, threaded=False,
                                  processes=CONF.cni_daemon.worker_num)
         except Exception:
             LOG.exception('Failed to start kuryr-daemon.')
             raise
 
+    def _check_failure(self):
+        with self.failure_count.get_lock():
+            if self.failure_count.value < CONF.cni_daemon.cni_failures_count:
+                self.failure_count.value += 1
+            else:
+                with self.healthy.get_lock():
+                    LOG.debug("Reporting maximun CNI ADD failures reached.")
+                    self.healthy.value = False
+
 
 class CNIDaemonServerService(cotyledon.Service):
     name = "server"
 
-    def __init__(self, worker_id, registry):
+    def __init__(self, worker_id, registry, healthy):
         super(CNIDaemonServerService, self).__init__(worker_id)
         self.run_queue_reading = False
         self.registry = registry
-        self.plugin = K8sCNIRegistryPlugin(registry)
-        self.server = DaemonServer(self.plugin)
+        self.healthy = healthy
+        self.plugin = k8s_cni_registry.K8sCNIRegistryPlugin(registry,
+                                                            self.healthy)
+        self.server = DaemonServer(self.plugin, self.healthy)
 
     def run(self):
         # NOTE(dulek): We might do a *lot* of pyroute2 operations, let's
@@ -239,11 +169,13 @@ class CNIDaemonServerService(cotyledon.Service):
 class CNIDaemonWatcherService(cotyledon.Service):
     name = "watcher"
 
-    def __init__(self, worker_id, registry):
+    def __init__(self, worker_id, registry, healthy):
         super(CNIDaemonWatcherService, self).__init__(worker_id)
         self.pipeline = None
         self.watcher = None
+        self.health_thread = None
         self.registry = registry
+        self.healthy = healthy
 
     def _get_nodename(self):
         # NOTE(dulek): At first try to get it using environment variable,
@@ -263,29 +195,49 @@ class CNIDaemonWatcherService(cotyledon.Service):
             "%(base)s/pods?fieldSelector=spec.nodeName=%(node_name)s" % {
                 'base': k_const.K8S_API_BASE,
                 'node_name': self._get_nodename()})
+        self.is_running = True
+        self.health_thread = threading.Thread(
+            target=self._start_watcher_health_checker)
+        self.health_thread.start()
         self.watcher.start()
 
-    def on_done(self, pod, vif):
-        pod_name = pod['metadata']['name']
-        vif_dict = vif.obj_to_primitive()
+    def _start_watcher_health_checker(self):
+        while self.is_running:
+            if not self.watcher.is_healthy():
+                LOG.debug("Reporting watcher not healthy.")
+                with self.healthy.get_lock():
+                    self.healthy.value = False
+            time.sleep(HEALTH_CHECKER_DELAY)
+
+    def on_done(self, pod, vifs):
+        pod_name = utils.get_pod_unique_name(pod)
+        vif_dict = {
+            ifname: vif.obj_to_primitive() for
+            ifname, vif in vifs.items()
+        }
         # NOTE(dulek): We need a lock when modifying shared self.registry dict
         #              to prevent race conditions with other processes/threads.
         with lockutils.lock(pod_name, external=True):
             if pod_name not in self.registry:
-                self.registry[pod_name] = {'pod': pod, 'vif': vif_dict,
+                self.registry[pod_name] = {'pod': pod, 'vifs': vif_dict,
                                            'containerid': None}
             else:
                 # NOTE(dulek): Only update vif if its status changed, we don't
                 #              need to care about other changes now.
-                old_vif = base.VersionedObject.obj_from_primitive(
-                    self.registry[pod_name]['vif'])
-                if old_vif.active != vif.active:
-                    pod_dict = self.registry[pod_name]
-                    pod_dict['vif'] = vif_dict
-                    self.registry[pod_name] = pod_dict
+                old_vifs = {
+                    ifname:
+                        base.VersionedObject.obj_from_primitive(vif_obj) for
+                        ifname, vif_obj in (
+                            self.registry[pod_name]['vifs'].items())
+                }
+                for iface in vifs.keys():
+                    if old_vifs[iface].active != vifs[iface].active:
+                        pod_dict = self.registry[pod_name]
+                        pod_dict['vifs'] = vif_dict
+                        self.registry[pod_name] = pod_dict
 
     def on_deleted(self, pod):
-        pod_name = pod['metadata']['name']
+        pod_name = utils.get_pod_unique_name(pod)
         try:
             if pod_name in self.registry:
                 # NOTE(dulek): del on dict is atomic as long as we use standard
@@ -297,10 +249,24 @@ class CNIDaemonWatcherService(cotyledon.Service):
             pass
 
     def terminate(self):
+        self.is_running = False
+        if self.health_thread:
+            self.health_thread.join()
         if self.watcher:
             self.watcher.stop()
 
 
+class CNIDaemonHealthServerService(cotyledon.Service):
+    name = "health"
+
+    def __init__(self, worker_id, healthy):
+        super(CNIDaemonHealthServerService, self).__init__(worker_id)
+        self.health_server = health.CNIHealthServer(healthy)
+
+    def run(self):
+        self.health_server.run()
+
+
 class CNIDaemonServiceManager(cotyledon.ServiceManager):
     def __init__(self):
         super(CNIDaemonServiceManager, self).__init__()
@@ -315,8 +281,10 @@ class CNIDaemonServiceManager(cotyledon.ServiceManager):
 
         self.manager = multiprocessing.Manager()
         registry = self.manager.dict()  # For Watcher->Server communication.
-        self.add(CNIDaemonWatcherService, workers=1, args=(registry,))
-        self.add(CNIDaemonServerService, workers=1, args=(registry,))
+        healthy = multiprocessing.Value(c_bool, True)
+        self.add(CNIDaemonWatcherService, workers=1, args=(registry, healthy,))
+        self.add(CNIDaemonServerService, workers=1, args=(registry, healthy,))
+        self.add(CNIDaemonHealthServerService, workers=1, args=(healthy,))
         self.register_hooks(on_terminate=self.terminate)
 
     def run(self):
