@@ -15,11 +15,18 @@
 
 K8S_API_BASE = '/api/v1'
 K8S_API_NAMESPACES = K8S_API_BASE + '/namespaces'
+K8S_API_CRD = '/apis/openstack.org/v1'
+K8S_API_POLICIES = '/apis/networking.k8s.io/v1/networkpolicies'
+
+K8S_API_NPWG_CRD = '/apis/k8s.cni.cncf.io/v1'
 
 K8S_OBJ_NAMESPACE = 'Namespace'
 K8S_OBJ_POD = 'Pod'
 K8S_OBJ_SERVICE = 'Service'
 K8S_OBJ_ENDPOINTS = 'Endpoints'
+K8S_OBJ_POLICY = 'NetworkPolicy'
+
+K8S_OBJ_KURYRNET = 'KuryrNet'
 
 K8S_POD_STATUS_PENDING = 'Pending'
 
@@ -27,6 +34,16 @@ K8S_ANNOTATION_PREFIX = 'openstack.org/kuryr'
 K8S_ANNOTATION_VIF = K8S_ANNOTATION_PREFIX + '-vif'
 K8S_ANNOTATION_LBAAS_SPEC = K8S_ANNOTATION_PREFIX + '-lbaas-spec'
 K8S_ANNOTATION_LBAAS_STATE = K8S_ANNOTATION_PREFIX + '-lbaas-state'
+K8S_ANNOTATION_NET_CRD = K8S_ANNOTATION_PREFIX + '-net-crd'
+K8S_ANNOTATION_LBAAS_RT_STATE = K8S_ANNOTATION_PREFIX + '-lbaas-route-state'
+K8S_ANNOTATION_LBAAS_RT_NOTIF = K8S_ANNOTATION_PREFIX + '-lbaas-route-notif'
+K8S_ANNOTATION_ROUTE_STATE = K8S_ANNOTATION_PREFIX + '-route-state'
+K8S_ANNOTATION_ROUTE_SPEC = K8S_ANNOTATION_PREFIX + '-route-spec'
+
+K8S_ANNOTATION_NPWG_PREFIX = 'k8s.v1.cni.cncf.io'
+K8S_ANNOTATION_NPWG_NETWORK = K8S_ANNOTATION_NPWG_PREFIX + '/networks'
+K8S_ANNOTATION_NPWG_CRD_SUBNET_ID = 'subnetId'
+K8S_ANNOTATION_NPWG_CRD_DRIVER_TYPE = 'driverType'
 
 K8S_OS_VIF_NOOP_PLUGIN = "noop"
 
@@ -34,6 +51,7 @@ CNI_EXCEPTION_CODE = 100
 CNI_TIMEOUT_CODE = 200
 
 KURYR_PORT_NAME = 'kuryr-pool-port'
+KURYR_L7_ROUTER_HTTP_PORT = '80'
 
 OCTAVIA_L2_MEMBER_MODE = "L2"
 OCTAVIA_L3_MEMBER_MODE = "L3"
@@ -43,3 +61,7 @@ VIF_POOL_POPULATE = '/populatePool'
 VIF_POOL_FREE = '/freePool'
 VIF_POOL_LIST = '/listPools'
 VIF_POOL_SHOW = '/showPool'
+
+DEFAULT_IFNAME = 'eth0'
+
+ADDITIONAL_IFNAME_PREFIX = 'eth'
