@@ -35,6 +35,7 @@ LOG = logging.getLogger(__name__)
 
 _ACTIVATION_TIMEOUT = CONF.neutron_defaults.lbaas_activation_timeout
 _SUPPORTED_LISTENER_PROT = ('HTTP', 'HTTPS', 'TCP')
+_L7_POLICY_ACT_REDIRECT_TO_POOL = 'REDIRECT_TO_POOL'
 
 
 class LBaaSv2Driver(base.LBaaSDriver):
@@ -45,9 +46,14 @@ class LBaaSv2Driver(base.LBaaSDriver):
         lbaas = clients.get_loadbalancer_client()
         return lbaas.cascading_capable
 
-    def ensure_loadbalancer(self, endpoints, project_id, subnet_id, ip,
-                            security_groups_ids, service_type):
-        name = "%(namespace)s/%(name)s" % endpoints['metadata']
+    def get_service_loadbalancer_name(self, namespace, svc_name):
+        return "%s/%s" % (namespace, svc_name)
+
+    def get_loadbalancer_pool_name(self, loadbalancer, namespace, svc_name):
+        return "%s/%s/%s" % (loadbalancer.name, namespace, svc_name)
+
+    def ensure_loadbalancer(self, name, project_id, subnet_id, ip,
+                            security_groups_ids=None, service_type=None):
         request = obj_lbaas.LBaaSLoadBalancer(
             name=name, project_id=project_id, subnet_id=subnet_id, ip=ip,
             security_groups=security_groups_ids)
@@ -59,17 +65,15 @@ class LBaaSv2Driver(base.LBaaSDriver):
             raise k_exc.ResourceNotReady(request)
 
         try:
-            self.ensure_security_groups(endpoints, response,
-                                        security_groups_ids, service_type)
+            if security_groups_ids is not None:
+                self._ensure_security_groups(response, service_type)
         except n_exc.NeutronClientException:
-            # NOTE(dulek): `endpoints` arguments on release_loadbalancer()
-            #              is ignored for some reason, so just pass None.
-            self.release_loadbalancer(None, response)
+            self.release_loadbalancer(response)
             raise
 
         return response
 
-    def release_loadbalancer(self, endpoints, loadbalancer):
+    def release_loadbalancer(self, loadbalancer):
         neutron = clients.get_neutron_client()
         lbaas = clients.get_loadbalancer_client()
         if lbaas.cascading_capable:
@@ -92,8 +96,7 @@ class LBaaSv2Driver(base.LBaaSDriver):
                     LOG.exception('Error when deleting loadbalancer security '
                                   'group. Leaving it orphaned.')
 
-    def ensure_security_groups(self, endpoints, loadbalancer,
-                               security_groups_ids, service_type):
+    def _ensure_security_groups(self, loadbalancer, service_type):
         # We only handle SGs for legacy LBaaSv2, Octavia handles it dynamically
         # according to listener ports.
         if loadbalancer.provider == const.NEUTRON_LBAAS_HAPROXY_PROVIDER:
@@ -126,7 +129,7 @@ class LBaaSv2Driver(base.LBaaSDriver):
                     neutron.delete_security_group(sg_id)
                 raise
 
-    def ensure_security_group_rules(self, endpoints, loadbalancer, listener):
+    def _ensure_lb_security_group_rule(self, loadbalancer, listener):
         sg_id = self._find_listeners_sg(loadbalancer)
         if sg_id:
             try:
@@ -146,13 +149,79 @@ class LBaaSv2Driver(base.LBaaSDriver):
                     LOG.exception('Failed when creating security group rule '
                                   'for listener %s.', listener.name)
 
-    def ensure_listener(self, endpoints, loadbalancer, protocol, port):
+    def _extend_lb_security_group_rules(self, loadbalancer, listener):
+        neutron = clients.get_neutron_client()
+        sg_id = self._get_vip_port(loadbalancer).get('security_groups')[0]
+        for sg in loadbalancer.security_groups:
+            try:
+                neutron.create_security_group_rule({
+                    'security_group_rule': {
+                        'direction': 'ingress',
+                        'port_range_min': listener.port,
+                        'port_range_max': listener.port,
+                        'protocol': listener.protocol,
+                        'security_group_id': sg_id,
+                        'remote_group_id': sg,
+                        'description': listener.name,
+                    },
+                })
+            except n_exc.NeutronClientException as ex:
+                if ex.status_code != requests.codes.conflict:
+                    LOG.exception('Failed when creating security group rule '
+                                  'for listener %s.', listener.name)
+
+        # ensure routes have access to the services
+        service_subnet_cidr = self._get_subnet_cidr(loadbalancer.subnet_id)
+        try:
+            # add access from service subnet
+            neutron.create_security_group_rule({
+                'security_group_rule': {
+                    'direction': 'ingress',
+                    'port_range_min': listener.port,
+                    'port_range_max': listener.port,
+                    'protocol': listener.protocol,
+                    'security_group_id': sg_id,
+                    'remote_ip_prefix': service_subnet_cidr,
+                    'description': listener.name,
+                },
+            })
+
+            # add access from worker node VM subnet for non-native route
+            # support
+            worker_subnet_id = CONF.pod_vif_nested.worker_nodes_subnet
+            if worker_subnet_id:
+                worker_subnet_cidr = self._get_subnet_cidr(worker_subnet_id)
+                neutron.create_security_group_rule({
+                    'security_group_rule': {
+                        'direction': 'ingress',
+                        'port_range_min': listener.port,
+                        'port_range_max': listener.port,
+                        'protocol': listener.protocol,
+                        'security_group_id': sg_id,
+                        'remote_ip_prefix': worker_subnet_cidr,
+                        'description': listener.name,
+                    },
+                })
+        except n_exc.NeutronClientException as ex:
+            if ex.status_code != requests.codes.conflict:
+                LOG.exception('Failed when creating security group rule '
+                              'to enable routes for listener %s.',
+                              listener.name)
+
+    def _ensure_security_group_rules(self, loadbalancer, listener,
+                                     service_type):
+        if loadbalancer.provider == const.NEUTRON_LBAAS_HAPROXY_PROVIDER:
+            self._ensure_lb_security_group_rule(loadbalancer, listener)
+        elif service_type == 'ClusterIP':
+            self._extend_lb_security_group_rules(loadbalancer, listener)
+
+    def ensure_listener(self, loadbalancer, protocol, port,
+                        service_type='ClusterIP'):
         if protocol not in _SUPPORTED_LISTENER_PROT:
             LOG.info("Protocol: %(prot)s: is not supported by LBaaSV2", {
                 'prot': protocol})
             return None
-        name = "%(namespace)s/%(name)s" % endpoints['metadata']
-        name += ":%s:%s" % (protocol, port)
+        name = "%s:%s:%s" % (loadbalancer.name, protocol, port)
         listener = obj_lbaas.LBaaSListener(name=name,
                                            project_id=loadbalancer.project_id,
                                            loadbalancer_id=loadbalancer.id,
@@ -162,11 +231,11 @@ class LBaaSv2Driver(base.LBaaSDriver):
                                           self._create_listener,
                                           self._find_listener)
 
-        self.ensure_security_group_rules(endpoints, loadbalancer, result)
+        self._ensure_security_group_rules(loadbalancer, result, service_type)
 
         return result
 
-    def release_listener(self, endpoints, loadbalancer, listener):
+    def release_listener(self, loadbalancer, listener):
         neutron = clients.get_neutron_client()
         lbaas = clients.get_loadbalancer_client()
         self._release(loadbalancer, listener,
@@ -184,7 +253,7 @@ class LBaaSv2Driver(base.LBaaSDriver):
                 LOG.warning('Cannot find SG rule for %s (%s) listener.',
                             listener.id, listener.name)
 
-    def ensure_pool(self, endpoints, loadbalancer, listener):
+    def ensure_pool(self, loadbalancer, listener):
         pool = obj_lbaas.LBaaSPool(name=listener.name,
                                    project_id=loadbalancer.project_id,
                                    loadbalancer_id=loadbalancer.id,
@@ -194,18 +263,32 @@ class LBaaSv2Driver(base.LBaaSDriver):
                                         self._create_pool,
                                         self._find_pool)
 
-    def release_pool(self, endpoints, loadbalancer, pool):
+    def ensure_pool_attached_to_lb(self, loadbalancer, namespace,
+                                   svc_name, protocol):
+        name = self.get_loadbalancer_pool_name(loadbalancer,
+                                               namespace, svc_name)
+        pool = obj_lbaas.LBaaSPool(name=name,
+                                   project_id=loadbalancer.project_id,
+                                   loadbalancer_id=loadbalancer.id,
+                                   listener_id=None,
+                                   protocol=protocol)
+        return self._ensure_provisioned(loadbalancer, pool,
+                                        self._create_pool,
+                                        self._find_pool_by_name)
+
+    def release_pool(self, loadbalancer, pool):
         lbaas = clients.get_loadbalancer_client()
         self._release(loadbalancer, pool,
                       lbaas.delete_lbaas_pool,
                       pool.id)
 
-    def ensure_member(self, endpoints, loadbalancer, pool,
-                      subnet_id, ip, port, target_ref):
-        name = "%(namespace)s/%(name)s" % target_ref
+    def ensure_member(self, loadbalancer, pool,
+                      subnet_id, ip, port, target_ref_namespace,
+                      target_ref_name):
+        name = ("%s/%s" % (target_ref_namespace, target_ref_name))
         name += ":%s" % port
         member = obj_lbaas.LBaaSMember(name=name,
-                                       project_id=pool.project_id,
+                                       project_id=loadbalancer.project_id,
                                        pool_id=pool.id,
                                        subnet_id=subnet_id,
                                        ip=ip,
@@ -214,13 +297,13 @@ class LBaaSv2Driver(base.LBaaSDriver):
                                         self._create_member,
                                         self._find_member)
 
-    def release_member(self, endpoints, loadbalancer, member):
+    def release_member(self, loadbalancer, member):
         lbaas = clients.get_loadbalancer_client()
         self._release(loadbalancer, member,
                       lbaas.delete_lbaas_member,
                       member.id, member.pool_id)
 
-    def _get_vip_port_id(self, loadbalancer):
+    def _get_vip_port(self, loadbalancer):
         neutron = clients.get_neutron_client()
         try:
             fixed_ips = ['subnet_id=%s' % str(loadbalancer.subnet_id),
@@ -231,10 +314,19 @@ class LBaaSv2Driver(base.LBaaSDriver):
             raise ex
 
         if ports['ports']:
-            return ports['ports'][0].get("id")
+            return ports['ports'][0]
 
         return None
 
+    def _get_subnet_cidr(self, subnet_id):
+        neutron = clients.get_neutron_client()
+        try:
+            subnet_obj = neutron.show_subnet(subnet_id)
+        except n_exc.NeutronClientException:
+            LOG.exception("Subnet %s CIDR not found!", subnet_id)
+            raise
+        return subnet_obj.get('subnet')['cidr']
+
     def _create_loadbalancer(self, loadbalancer):
         lbaas = clients.get_loadbalancer_client()
         response = lbaas.create_loadbalancer({'loadbalancer': {
@@ -243,7 +335,7 @@ class LBaaSv2Driver(base.LBaaSDriver):
             'vip_address': str(loadbalancer.ip),
             'vip_subnet_id': loadbalancer.subnet_id}})
         loadbalancer.id = response['loadbalancer']['id']
-        loadbalancer.port_id = self._get_vip_port_id(loadbalancer)
+        loadbalancer.port_id = self._get_vip_port(loadbalancer).get("id")
         loadbalancer.provider = response['loadbalancer']['provider']
         return loadbalancer
 
@@ -257,7 +349,7 @@ class LBaaSv2Driver(base.LBaaSDriver):
 
         try:
             loadbalancer.id = response['loadbalancers'][0]['id']
-            loadbalancer.port_id = self._get_vip_port_id(loadbalancer)
+            loadbalancer.port_id = self._get_vip_port(loadbalancer).get("id")
             loadbalancer.provider = response['loadbalancers'][0]['provider']
         except (KeyError, IndexError):
             return None
@@ -319,7 +411,7 @@ class LBaaSv2Driver(base.LBaaSDriver):
             loadbalancer_id=pool.loadbalancer_id,
             protocol=pool.protocol, lb_algorithm=lb_algorithm)
         bogus_pool_ids = [p['id'] for p in pools.get('pools')
-                          if not p['listeners']]
+                          if not p['listeners'] and pool.name == p['name']]
         for pool_id in bogus_pool_ids:
             try:
                 LOG.debug("Removing bogus pool %(id)s %(pool)s", {
@@ -328,7 +420,7 @@ class LBaaSv2Driver(base.LBaaSDriver):
             except (n_exc.NotFound, n_exc.StateInvalidClient):
                 pass
 
-    def _find_pool(self, pool):
+    def _find_pool(self, pool, by_listener=True):
         lbaas = clients.get_loadbalancer_client()
         response = lbaas.list_lbaas_pools(
             name=pool.name,
@@ -337,14 +429,22 @@ class LBaaSv2Driver(base.LBaaSDriver):
             protocol=pool.protocol)
 
         try:
-            pools = [p for p in response['pools']
-                     if pool.listener_id in {l['id'] for l in p['listeners']}]
+            if by_listener:
+                pools = [p for p in response['pools']
+                         if pool.listener_id
+                         in {l['id'] for l in p['listeners']}]
+            else:
+                pools = [p for p in response['pools']
+                         if pool.name == p['name']]
+
             pool.id = pools[0]['id']
         except (KeyError, IndexError):
             return None
-
         return pool
 
+    def _find_pool_by_name(self, pool):
+        return self._find_pool(pool, by_listener=False)
+
     def _create_member(self, member):
         lbaas = clients.get_loadbalancer_client()
         response = lbaas.create_lbaas_member(member.pool_id, {'member': {
@@ -374,10 +474,11 @@ class LBaaSv2Driver(base.LBaaSDriver):
         return member
 
     def _ensure(self, obj, create, find):
+        # TODO(yboaron): change the create/find order.
         try:
             result = create(obj)
             LOG.debug("Created %(obj)s", {'obj': result})
-        except n_exc.Conflict:
+        except (n_exc.Conflict, n_exc.InternalServerError):
             result = find(obj)
             if result:
                 LOG.debug("Found %(obj)s", {'obj': result})
@@ -453,3 +554,148 @@ class LBaaSv2Driver(base.LBaaSDriver):
                           loadbalancer.name)
 
         return None
+
+    def get_lb_by_uuid(self, lb_uuid):
+        lbaas = clients.get_loadbalancer_client()
+        response = lbaas.show_loadbalancer(lb_uuid)
+        try:
+            return obj_lbaas.LBaaSLoadBalancer(
+                id=response['loadbalancer']['id'],
+                port_id=response['loadbalancer']['vip_port_id'],
+                name=response['loadbalancer']['name'],
+                project_id=response['loadbalancer']['project_id'],
+                subnet_id=response['loadbalancer']['vip_subnet_id'],
+                ip=response['loadbalancer']['vip_address'],
+                security_groups=None,
+                provider=response['loadbalancer']['provider'])
+        except (KeyError, IndexError):
+            LOG.debug("Couldn't find loadbalancer with uuid=%s", lb_uuid)
+        return None
+
+    def get_pool_by_name(self, pool_name, project_id):
+        lbaas = clients.get_loadbalancer_client()
+
+        # NOTE(yboaron): pool_name should be constructed using
+        # get_loadbalancer_pool_name function, which means that pool's name
+        # is unique
+
+        pools_list = lbaas.list_lbaas_pools(
+            project_id=project_id)
+        for entry in pools_list['pools']:
+            if not entry:
+                continue
+            if entry['name'] == pool_name:
+                listener_id = (entry['listeners'][0]['id'] if
+                               entry['listeners'] else None)
+                return obj_lbaas.LBaaSPool(
+                    name=entry['name'], project_id=entry['project_id'],
+                    loadbalancer_id=entry['loadbalancers'][0]['id'],
+                    listener_id=listener_id,
+                    protocol=entry['protocol'], id=entry['id'])
+        return None
+
+    def ensure_l7_policy(self, namespace, route_name,
+                         loadbalancer, pool,
+                         listener_id):
+        name = namespace + route_name
+        l7_policy = obj_lbaas.LBaaSL7Policy(name=name,
+                                            project_id=pool.project_id,
+                                            listener_id=listener_id,
+                                            redirect_pool_id=pool.id)
+
+        return self._ensure_provisioned(
+            loadbalancer, l7_policy, self._create_l7_policy,
+            self._find_l7_policy)
+
+    def release_l7_policy(self, loadbalancer, l7_policy):
+        lbaas = clients.get_loadbalancer_client()
+        self._release(
+            loadbalancer, l7_policy, lbaas.delete_lbaas_l7policy,
+            l7_policy.id)
+
+    def _create_l7_policy(self, l7_policy):
+        lbaas = clients.get_loadbalancer_client()
+        response = lbaas.create_lbaas_l7policy({'l7policy': {
+            'action': _L7_POLICY_ACT_REDIRECT_TO_POOL,
+            'listener_id': l7_policy.listener_id,
+            'name': l7_policy.name,
+            'project_id': l7_policy.project_id,
+            'redirect_pool_id': l7_policy.redirect_pool_id}})
+        l7_policy.id = response['l7policy']['id']
+        return l7_policy
+
+    def _find_l7_policy(self, l7_policy):
+        lbaas = clients.get_loadbalancer_client()
+        response = lbaas.list_lbaas_l7policies(
+            name=l7_policy.name,
+            project_id=l7_policy.project_id,
+            redirect_pool_id=l7_policy.redirect_pool_id,
+            listener_id=l7_policy.listener_id)
+        try:
+            l7_policy.id = response['l7policies'][0]['id']
+        except (KeyError, IndexError):
+            return None
+        return l7_policy
+
+    def ensure_l7_rule(self, loadbalancer, l7_policy, compare_type,
+                       type, value):
+
+        l7_rule = obj_lbaas.LBaaSL7Rule(
+            compare_type=compare_type, l7policy_id=l7_policy.id,
+            type=type, value=value)
+        return self._ensure_provisioned(
+            loadbalancer, l7_rule, self._create_l7_rule,
+            self._find_l7_rule)
+
+    def _create_l7_rule(self, l7_rule):
+        lbaas = clients.get_loadbalancer_client()
+        response = lbaas.create_lbaas_l7rule(
+            l7_rule.l7policy_id,
+            {'rule': {'compare_type': l7_rule.compare_type,
+                      'type': l7_rule.type,
+                      'value': l7_rule.value}})
+        l7_rule.id = response['rule']['id']
+        return l7_rule
+
+    def _find_l7_rule(self, l7_rule):
+        lbaas = clients.get_loadbalancer_client()
+        response = lbaas.list_lbaas_l7rules(
+            l7_rule.l7policy_id,
+            type=l7_rule.type,
+            value=l7_rule.value,
+            compare_type=l7_rule.compare_type)
+        try:
+            l7_rule.id = response['rules'][0]['id']
+        except (KeyError, IndexError):
+            return None
+        return l7_rule
+
+    def release_l7_rule(self, loadbalancer, l7_rule):
+        lbaas = clients.get_loadbalancer_client()
+        self._release(
+            loadbalancer, l7_rule, lbaas.delete_lbaas_l7rule,
+            l7_rule.id, l7_rule.l7policy_id)
+
+    def update_l7_rule(self, l7_rule, new_value):
+        lbaas = clients.get_loadbalancer_client()
+        try:
+            lbaas.update_lbaas_l7rule(
+                l7_rule.id, l7_rule.l7policy_id,
+                {'rule': {'value': new_value}})
+
+        except n_exc.NeutronClientException as ex:
+            LOG.error("Failed to update l7_rule- id=%s ",
+                      l7_rule.id)
+            raise ex
+
+    def is_pool_used_by_other_l7policies(self, l7policy, pool):
+        lbaas = clients.get_loadbalancer_client()
+        l7policy_list = lbaas.list_lbaas_l7policies(
+            project_id=l7policy.project_id)
+        for entry in l7policy_list['l7policies']:
+            if not entry:
+                continue
+            if (entry['redirect_pool_id'] == pool.id and
+                    entry['id'] != l7policy.id):
+                return True
+        return False
