@@ -104,6 +104,32 @@ class K8sClient(object):
             return response.json().get('status')
         raise exc.K8sClientException(response.text)
 
+    def post(self, path, body):
+        LOG.debug("Post %(path)s: %(body)s", {'path': path, 'body': body})
+        url = self._base_url + path
+        header = {'Content-Type': 'application/json'}
+        if self.token:
+            header.update({'Authorization': 'Bearer %s' % self.token})
+
+        response = requests.post(url, json=body, cert=self.cert,
+                                 verify=self.verify_server, headers=header)
+        if response.ok:
+            return response.json()
+        raise exc.K8sClientException(response)
+
+    def delete(self, path):
+        LOG.debug("Delete %(path)s", {'path': path})
+        url = self._base_url + path
+        header = {'Content-Type': 'application/json'}
+        if self.token:
+            header.update({'Authorization': 'Bearer %s' % self.token})
+
+        response = requests.delete(url, cert=self.cert,
+                                   verify=self.verify_server, headers=header)
+        if response.ok:
+            return response.json()
+        raise exc.K8sClientException(response)
+
     def annotate(self, path, annotations, resource_version=None):
         """Pushes a resource annotation to the K8s API resource
 
