@@ -350,16 +350,16 @@ class TestLBaaSSpecHandler(test_base.TestCase):
 
 class FakeLBaaSDriver(drv_base.LBaaSDriver):
 
-    def ensure_loadbalancer(self, endpoints, project_id, subnet_id, ip,
+    def ensure_loadbalancer(self, name, project_id, subnet_id, ip,
                             security_groups_ids, service_type):
-        name = str(ip)
         return obj_lbaas.LBaaSLoadBalancer(name=name,
                                            project_id=project_id,
                                            subnet_id=subnet_id,
                                            ip=ip,
                                            id=uuidutils.generate_uuid())
 
-    def ensure_listener(self, endpoints, loadbalancer, protocol, port):
+    def ensure_listener(self, loadbalancer, protocol, port,
+                        service_type='ClusterIP'):
         if protocol not in _SUPPORTED_LISTENER_PROT:
             return None
 
@@ -371,7 +371,7 @@ class FakeLBaaSDriver(drv_base.LBaaSDriver):
                                        port=port,
                                        id=uuidutils.generate_uuid())
 
-    def ensure_pool(self, endpoints, loadbalancer, listener):
+    def ensure_pool(self, loadbalancer, listener):
         return obj_lbaas.LBaaSPool(name=listener.name,
                                    project_id=loadbalancer.project_id,
                                    loadbalancer_id=loadbalancer.id,
@@ -379,8 +379,9 @@ class FakeLBaaSDriver(drv_base.LBaaSDriver):
                                    protocol=listener.protocol,
                                    id=uuidutils.generate_uuid())
 
-    def ensure_member(self, endpoints, loadbalancer, pool, subnet_id, ip, port,
-                      target_ref):
+    def ensure_member(self, loadbalancer, pool, subnet_id, ip, port,
+                      target_ref_namespace, target_ref_name
+                      ):
         name = "%s:%s:%s" % (loadbalancer.name, ip, port)
         return obj_lbaas.LBaaSMember(name=name,
                                      project_id=pool.project_id,
@@ -390,16 +391,56 @@ class FakeLBaaSDriver(drv_base.LBaaSDriver):
                                      port=port,
                                      id=uuidutils.generate_uuid())
 
-    def release_loadbalancer(self, endpoints, loadbalancer):
+    def release_loadbalancer(self, loadbalancer):
         pass
 
-    def release_listener(self, endpoints, loadbalancer, listener):
+    def release_listener(self, loadbalancer, listener):
         pass
 
-    def release_pool(self, endpoints, loadbalancer, pool):
+    def release_pool(self, loadbalancer, pool):
         pass
 
-    def release_member(self, endpoints, loadbalancer, member):
+    def release_member(self, loadbalancer, member):
+        pass
+
+    def get_service_loadbalancer_name(self, namespace, svc_name):
+        return "%s/%s" % (namespace, svc_name)
+
+    def get_loadbalancer_pool_name(self, lb_name, namespace, svc_name):
+        return "%s/%s/%s" % (lb_name, namespace, svc_name)
+
+    def ensure_l7_policy(self, namespace, route_name,
+                         loadbalancer, pool,
+                         listener_id):
+        pass
+
+    def release_l7_policy(self, loadbalancer, l7_policy):
+        pass
+
+    def ensure_l7_rule(self, loadbalancer, l7_policy, compare_type,
+                       type, value):
+        pass
+
+    def release_l7_rule(self, loadbalancer, l7_rule):
+        pass
+
+    def update_l7_rule(self, l7_rule, new_value):
+        pass
+
+    def ensure_pool_attached_to_lb(self, loadbalancer, namespace,
+                                   svc_name, protocol):
+        return obj_lbaas.LBaaSPool(id=uuidutils.generate_uuid(),
+                                   loadbalancer_id=loadbalancer.id,
+                                   project_id=loadbalancer.project_id,
+                                   protocol=protocol)
+
+    def get_pool_by_name(self, pool_name, project_id):
+        pass
+
+    def get_lb_by_uuid(self, vip):
+        pass
+
+    def is_pool_used_by_other_l7policies(l7policy, pool):
         pass
 
 
@@ -544,7 +585,7 @@ class TestLoadBalancerHandler(test_base.TestCase):
         h_lbaas.LoadBalancerHandler.on_deleted(m_handler, endpoints)
 
         m_handler._drv_lbaas.release_loadbalancer.assert_called_once_with(
-            endpoints=endpoints, loadbalancer=lbaas_state.loadbalancer)
+            loadbalancer=lbaas_state.loadbalancer)
         m_handler._drv_service_pub_ip.release_pub_ip.assert_called_once_with(
             lbaas_state.service_pub_ip_info)
 
@@ -615,22 +656,21 @@ class TestLoadBalancerHandler(test_base.TestCase):
         self.assertEqual(subnet_id, observed_subnet_id)
 
     def _generate_lbaas_state(self, vip, targets, project_id, subnet_id):
-        endpoints = mock.sentinel.endpoints
+        name = 'DUMMY_NAME'
         drv = FakeLBaaSDriver()
         lb = drv.ensure_loadbalancer(
-            endpoints, project_id, subnet_id, vip, None, 'ClusterIP')
+            name, project_id, subnet_id, vip, None, 'ClusterIP')
         listeners = {}
         pools = {}
         members = {}
         for ip, (listen_port, target_port) in targets.items():
             lsnr = listeners.setdefault(listen_port, drv.ensure_listener(
-                endpoints, lb, 'TCP', listen_port))
-            pool = pools.setdefault(listen_port, drv.ensure_pool(
-                endpoints, lb, lsnr))
+                lb, 'TCP', listen_port))
+            pool = pools.setdefault(listen_port, drv.ensure_pool(lb, lsnr))
             members.setdefault((ip, listen_port, target_port),
-                               drv.ensure_member(endpoints, lb, pool,
+                               drv.ensure_member(lb, pool,
                                                  subnet_id, ip,
-                                                 target_port, None))
+                                                 target_port, None, None))
         return obj_lbaas.LBaaSState(
             loadbalancer=lb,
             listeners=list(listeners.values()),
@@ -638,7 +678,7 @@ class TestLoadBalancerHandler(test_base.TestCase):
             members=list(members.values()))
 
     def _generate_lbaas_spec(self, vip, targets, project_id,
-                             subnet_id, prot='TCP'):
+                             subnet_id, prot='TCP', lbaas_type='ClusterIP'):
         return obj_lbaas.LBaaSServiceSpec(
             ip=vip,
             project_id=project_id,
@@ -646,7 +686,8 @@ class TestLoadBalancerHandler(test_base.TestCase):
             ports=[obj_lbaas.LBaaSPortSpec(name=str(port),
                                            protocol=prot,
                                            port=port)
-                   for port in set(t[0] for t in targets.values())])
+                   for port in set(t[0] for t in targets.values())],
+            type=lbaas_type)
 
     def _generate_endpoints(self, targets):
         def _target_to_port(item):
@@ -657,6 +698,10 @@ class TestLoadBalancerHandler(test_base.TestCase):
             for p, grp in itertools.groupby(
                 sorted(targets.items()), _target_to_port)]
         return {
+            'metadata': {
+                'name': 'ep_name',
+                'namespace': 'default'
+            },
             'subsets': [
                 {
                     'addresses': [
