@@ -15,10 +15,13 @@
 
 import mock
 
+from os_vif import objects as os_obj
+
 from kuryr_kubernetes import constants as k_const
 from kuryr_kubernetes.controller.drivers import base as drivers
 from kuryr_kubernetes.controller.handlers import vif as h_vif
 from kuryr_kubernetes import exceptions as k_exc
+from kuryr_kubernetes.objects import vif
 from kuryr_kubernetes.tests import base as test_base
 
 
@@ -30,9 +33,12 @@ class TestVIFHandler(test_base.TestCase):
         self._project_id = mock.sentinel.project_id
         self._subnets = mock.sentinel.subnets
         self._security_groups = mock.sentinel.security_groups
-        self._vif = mock.Mock()
+        self._vif = os_obj.vif.VIFBase()
         self._vif.active = True
         self._vif_serialized = mock.sentinel.vif_serialized
+        self._multi_vif_drv = mock.MagicMock(spec=drivers.MultiVIFDriver)
+        self._additioan_vifs = []
+        self._state = vif.PodState(default_vif=self._vif)
 
         self._pod_version = mock.sentinel.pod_version
         self._pod_link = mock.sentinel.pod_link
@@ -51,29 +57,37 @@ class TestVIFHandler(test_base.TestCase):
         self._handler._drv_vif = mock.Mock(spec=drivers.PodVIFDriver)
         self._handler._drv_vif_pool = mock.MagicMock(
             spec=drivers.VIFPoolDriver)
+        self._handler._drv_multi_vif = [self._multi_vif_drv]
 
         self._get_project = self._handler._drv_project.get_project
         self._get_subnets = self._handler._drv_subnets.get_subnets
         self._get_security_groups = self._handler._drv_sg.get_security_groups
-        self._set_vif_driver = self._handler._drv_vif_pool.set_vif_driver
+        self._set_vifs_driver = self._handler._drv_vif_pool.set_vif_driver
         self._request_vif = self._handler._drv_vif_pool.request_vif
         self._release_vif = self._handler._drv_vif_pool.release_vif
         self._activate_vif = self._handler._drv_vif_pool.activate_vif
-        self._get_vif = self._handler._get_vif
-        self._set_vif = self._handler._set_vif
+        self._get_pod_state = self._handler._get_pod_state
+        self._set_pod_state = self._handler._set_pod_state
         self._is_host_network = self._handler._is_host_network
         self._is_pending_node = self._handler._is_pending_node
+        self._request_additional_vifs = \
+            self._multi_vif_drv.request_additional_vifs
 
         self._request_vif.return_value = self._vif
-        self._get_vif.return_value = self._vif
+        self._request_additional_vifs.return_value = self._additioan_vifs
+        self._get_pod_state.return_value = self._state
         self._is_host_network.return_value = False
         self._is_pending_node.return_value = True
         self._get_project.return_value = self._project_id
         self._get_subnets.return_value = self._subnets
         self._get_security_groups.return_value = self._security_groups
-        self._set_vif_driver.return_value = mock.Mock(
+        self._set_vifs_driver.return_value = mock.Mock(
             spec=drivers.PodVIFDriver)
 
+        self._handler._drv_for_vif = h_vif.VIFHandler._drv_for_vif.__get__(
+            self._handler)
+
+    @mock.patch.object(drivers.MultiVIFDriver, 'get_enabled_drivers')
     @mock.patch.object(drivers.VIFPoolDriver, 'set_vif_driver')
     @mock.patch.object(drivers.VIFPoolDriver, 'get_instance')
     @mock.patch.object(drivers.PodVIFDriver, 'get_instance')
@@ -82,17 +96,19 @@ class TestVIFHandler(test_base.TestCase):
     @mock.patch.object(drivers.PodProjectDriver, 'get_instance')
     def test_init(self, m_get_project_driver, m_get_subnets_driver,
                   m_get_sg_driver, m_get_vif_driver, m_get_vif_pool_driver,
-                  m_set_vif_driver):
+                  m_set_vifs_driver, m_get_multi_vif_drivers):
         project_driver = mock.sentinel.project_driver
         subnets_driver = mock.sentinel.subnets_driver
         sg_driver = mock.sentinel.sg_driver
         vif_driver = mock.sentinel.vif_driver
         vif_pool_driver = mock.Mock(spec=drivers.VIFPoolDriver)
+        multi_vif_drivers = [mock.MagicMock(spec=drivers.MultiVIFDriver)]
         m_get_project_driver.return_value = project_driver
         m_get_subnets_driver.return_value = subnets_driver
         m_get_sg_driver.return_value = sg_driver
         m_get_vif_driver.return_value = vif_driver
         m_get_vif_pool_driver.return_value = vif_pool_driver
+        m_get_multi_vif_drivers.return_value = multi_vif_drivers
 
         handler = h_vif.VIFHandler()
 
@@ -100,6 +116,7 @@ class TestVIFHandler(test_base.TestCase):
         self.assertEqual(subnets_driver, handler._drv_subnets)
         self.assertEqual(sg_driver, handler._drv_sg)
         self.assertEqual(vif_pool_driver, handler._drv_vif_pool)
+        self.assertEqual(multi_vif_drivers, handler._drv_multi_vif)
 
     def test_is_host_network(self):
         self._pod['spec']['hostNetwork'] = True
@@ -131,62 +148,86 @@ class TestVIFHandler(test_base.TestCase):
     def test_on_present(self):
         h_vif.VIFHandler.on_present(self._handler, self._pod)
 
-        self._get_vif.assert_called_once_with(self._pod)
+        self._get_pod_state.assert_called_once_with(self._pod)
         self._request_vif.assert_not_called()
+        self._request_additional_vifs.assert_not_called()
         self._activate_vif.assert_not_called()
-        self._set_vif.assert_not_called()
+        self._set_pod_state.assert_not_called()
 
     def test_on_present_host_network(self):
         self._is_host_network.return_value = True
 
         h_vif.VIFHandler.on_present(self._handler, self._pod)
 
-        self._get_vif.assert_not_called()
+        self._get_pod_state.assert_not_called()
         self._request_vif.assert_not_called()
+        self._request_additional_vifs.assert_not_called()
         self._activate_vif.assert_not_called()
-        self._set_vif.assert_not_called()
+        self._set_pod_state.assert_not_called()
 
     def test_on_present_not_pending(self):
         self._is_pending_node.return_value = False
 
         h_vif.VIFHandler.on_present(self._handler, self._pod)
 
-        self._get_vif.assert_not_called()
+        self._get_pod_state.assert_not_called()
         self._request_vif.assert_not_called()
+        self._request_additional_vifs.assert_not_called()
         self._activate_vif.assert_not_called()
-        self._set_vif.assert_not_called()
+        self._set_pod_state.assert_not_called()
 
     def test_on_present_activate(self):
         self._vif.active = False
 
         h_vif.VIFHandler.on_present(self._handler, self._pod)
 
-        self._get_vif.assert_called_once_with(self._pod)
+        self._get_pod_state.assert_called_once_with(self._pod)
         self._activate_vif.assert_called_once_with(self._pod, self._vif)
-        self._set_vif.assert_called_once_with(self._pod, self._vif)
+        self._set_pod_state.assert_called_once_with(self._pod, self._state)
         self._request_vif.assert_not_called()
+        self._request_additional_vifs.assert_not_called()
 
     def test_on_present_create(self):
-        self._get_vif.return_value = None
+        self._get_pod_state.return_value = None
 
         h_vif.VIFHandler.on_present(self._handler, self._pod)
 
-        self._get_vif.assert_called_once_with(self._pod)
+        self._get_pod_state.assert_called_once_with(self._pod)
         self._request_vif.assert_called_once_with(
             self._pod, self._project_id, self._subnets, self._security_groups)
-        self._set_vif.assert_called_once_with(self._pod, self._vif)
+        self._request_additional_vifs.assert_called_once_with(
+            self._pod, self._project_id, self._security_groups)
+        self._set_pod_state.assert_called_once_with(self._pod, self._state)
+        self._activate_vif.assert_not_called()
+
+    def test_on_present_create_with_additional_vifs(self):
+        self._get_pod_state.return_value = None
+        additional_vif = os_obj.vif.VIFBase()
+        self._state.additional_vifs = {'eth1': additional_vif}
+        self._request_additional_vifs.return_value = [additional_vif]
+
+        h_vif.VIFHandler.on_present(self._handler, self._pod)
+
+        self._get_pod_state.assert_called_once_with(self._pod)
+        self._request_vif.assert_called_once_with(
+            self._pod, self._project_id, self._subnets, self._security_groups)
+        self._request_additional_vifs.assert_called_once_with(
+            self._pod, self._project_id, self._security_groups)
+        self._set_pod_state.assert_called_once_with(self._pod, self._state)
         self._activate_vif.assert_not_called()
 
     def test_on_present_rollback(self):
-        self._get_vif.return_value = None
-        self._set_vif.side_effect = k_exc.K8sClientException
+        self._get_pod_state.return_value = None
+        self._set_pod_state.side_effect = k_exc.K8sClientException
 
         h_vif.VIFHandler.on_present(self._handler, self._pod)
 
-        self._get_vif.assert_called_once_with(self._pod)
+        self._get_pod_state.assert_called_once_with(self._pod)
         self._request_vif.assert_called_once_with(
             self._pod, self._project_id, self._subnets, self._security_groups)
-        self._set_vif.assert_called_once_with(self._pod, self._vif)
+        self._request_additional_vifs.assert_called_once_with(
+            self._pod, self._project_id, self._security_groups)
+        self._set_pod_state.assert_called_once_with(self._pod, self._state)
         self._release_vif.assert_called_once_with(self._pod, self._vif,
                                                   self._project_id,
                                                   self._security_groups)
@@ -195,23 +236,37 @@ class TestVIFHandler(test_base.TestCase):
     def test_on_deleted(self):
         h_vif.VIFHandler.on_deleted(self._handler, self._pod)
 
-        self._get_vif.assert_called_once_with(self._pod)
+        self._get_pod_state.assert_called_once_with(self._pod)
         self._release_vif.assert_called_once_with(self._pod, self._vif,
                                                   self._project_id,
                                                   self._security_groups)
 
+    def test_on_deleted_with_additional_vifs(self):
+        additional_vif = os_obj.vif.VIFBase()
+        self._state.additional_vifs = {'eth1': additional_vif}
+        self._get_pod_state.return_value = self._state
+
+        h_vif.VIFHandler.on_deleted(self._handler, self._pod)
+
+        self._release_vif.assert_any_call(self._pod, self._vif,
+                                          self._project_id,
+                                          self._security_groups)
+        self._release_vif.assert_any_call(self._pod, additional_vif,
+                                          self._project_id,
+                                          self._security_groups)
+
     def test_on_deleted_host_network(self):
         self._is_host_network.return_value = True
 
         h_vif.VIFHandler.on_deleted(self._handler, self._pod)
 
-        self._get_vif.assert_not_called()
+        self._get_pod_state.assert_not_called()
         self._release_vif.assert_not_called()
 
     def test_on_deleted_no_annotation(self):
-        self._get_vif.return_value = None
+        self._get_pod_state.return_value = None
 
         h_vif.VIFHandler.on_deleted(self._handler, self._pod)
 
-        self._get_vif.assert_called_once_with(self._pod)
+        self._get_pod_state.assert_called_once_with(self._pod)
         self._release_vif.assert_not_called()
