@@ -280,6 +280,33 @@ class TestK8sClient(test_base.TestCase):
                                         headers=mock.ANY,
                                         cert=(None, None), verify=False)
 
+    @mock.patch('itertools.count')
+    @mock.patch('requests.patch')
+    def test_annotate_resource_not_found(self, m_patch, m_count):
+        m_count.return_value = list(range(1, 5))
+        path = '/test'
+        annotations = {'a1': 'v1', 'a2': 'v2'}
+        resource_version = "123"
+        annotate_obj = {'metadata': {
+            'annotations': annotations,
+            'resourceVersion': resource_version}}
+        annotate_data = jsonutils.dumps(annotate_obj, sort_keys=True)
+
+        m_resp_not_found = mock.MagicMock()
+        m_resp_not_found.ok = False
+        m_resp_not_found.status_code = requests.codes.not_found
+        m_patch.return_value = m_resp_not_found
+
+        self.assertRaises(exc.K8sResourceNotFound,
+                          self.client.annotate,
+                          path,
+                          annotations,
+                          resource_version=resource_version)
+        m_patch.assert_called_once_with(self.base_url + path,
+                                        data=annotate_data,
+                                        headers=mock.ANY,
+                                        cert=(None, None), verify=False)
+
     @mock.patch('requests.get')
     def test_watch(self, m_get):
         path = '/test'
@@ -313,3 +340,57 @@ class TestK8sClient(test_base.TestCase):
 
         self.assertRaises(exc.K8sClientException, next,
                           self.client.watch(path))
+
+    @mock.patch('requests.post')
+    def test_post(self, m_post):
+        path = '/test'
+        body = {'test': 'body'}
+        ret = {'test': 'value'}
+
+        m_resp = mock.MagicMock()
+        m_resp.ok = True
+        m_resp.json.return_value = ret
+        m_post.return_value = m_resp
+
+        self.assertEqual(ret, self.client.post(path, body))
+        m_post.assert_called_once_with(self.base_url + path, json=body,
+                                       headers=mock.ANY, cert=(None, None),
+                                       verify=False)
+
+    @mock.patch('requests.post')
+    def test_post_exception(self, m_post):
+        path = '/test'
+        body = {'test': 'body'}
+
+        m_resp = mock.MagicMock()
+        m_resp.ok = False
+        m_post.return_value = m_resp
+
+        self.assertRaises(exc.K8sClientException,
+                          self.client.post, path, body)
+
+    @mock.patch('requests.delete')
+    def test_delete(self, m_delete):
+        path = '/test'
+        ret = {'test': 'value'}
+
+        m_resp = mock.MagicMock()
+        m_resp.ok = True
+        m_resp.json.return_value = ret
+        m_delete.return_value = m_resp
+
+        self.assertEqual(ret, self.client.delete(path))
+        m_delete.assert_called_once_with(self.base_url + path,
+                                         headers=mock.ANY, cert=(None, None),
+                                         verify=False)
+
+    @mock.patch('requests.delete')
+    def test_delete_exception(self, m_delete):
+        path = '/test'
+
+        m_resp = mock.MagicMock()
+        m_resp.ok = False
+        m_delete.return_value = m_resp
+
+        self.assertRaises(exc.K8sClientException,
+                          self.client.delete, path)
