@@ -20,6 +20,8 @@ source "$DIR/../devstack/lib/kuryr_kubernetes"
 OUTPUT_DIR=${1:-.}
 CONTROLLER_CONF_PATH=${2:-""}
 CNI_CONF_PATH=${3:-$CONTROLLER_CONF_PATH}
+CA_CERTIFICATE_PATH=${CA_CERTIFICATE_PATH:-""}
+CA_CERTIFICATE_PATH=${4:-$CA_CERTIFICATE_PATH}
 
 if [ -z $CONTROLLER_CONF_PATH ]; then
     api_root=${KURYR_K8S_API_ROOT:-https://127.0.0.1:6443}
@@ -36,6 +38,7 @@ if [ -z $CONTROLLER_CONF_PATH ]; then
     worker_nodes_subnet=${KURYR_K8S_WORKER_NODES_SUBNET}
     binding_driver=${KURYR_K8S_BINDING_DRIVER:-kuryr.lib.binding.drivers.vlan}
     binding_iface=${KURYR_K8S_BINDING_IFACE:-eth0}
+    pod_subnet_pool=${KURYR_NEUTRON_DEFAULT_SUBNETPOOL_ID}
 
     CONTROLLER_CONF_PATH="${OUTPUT_DIR}/kuryr.conf"
     rm -f $CONTROLLER_CONF_PATH
@@ -61,6 +64,8 @@ service_subnet = $service_subnet_id
 pod_security_groups = $pod_sg
 pod_subnet = $pod_subnet_id
 project = $k8s_project_id
+[namespace_subnet]
+pod_subnet_pool = $pod_subnet_pool
 EOF
 
     if [ ! -z $binding_driver ]; then
@@ -90,6 +95,9 @@ ssl_verify_server_crt = true
 helper_command=privsep-helper
 [vif_plug_linux_bridge_privileged]
 helper_command=privsep-helper
+[cni_daemon]
+docker_mode = true
+netns_proc_dir = /host_proc
 EOF
 
     if [ ! -z $binding_driver ]; then
@@ -103,8 +111,10 @@ EOF
     fi
 fi
 
+generate_kuryr_certificates_secret $OUTPUT_DIR $CA_CERTIFICATE_PATH
 generate_kuryr_configmap $OUTPUT_DIR $CONTROLLER_CONF_PATH $CNI_CONF_PATH
 generate_kuryr_service_account $OUTPUT_DIR
 health_server_port=${KURYR_HEALTH_SERVER_PORT:-8082}
 generate_controller_deployment $OUTPUT_DIR $health_server_port
-generate_cni_daemon_set $OUTPUT_DIR
+cni_health_server_port=${KURYR_CNI_HEALTH_SERVER_PORT:-8090}
+generate_cni_daemon_set $OUTPUT_DIR $cni_health_server_port
